@@ -6,6 +6,9 @@ import numpy as np
 from scipy.ndimage import percentile_filter
 from scipy.stats import norm
 
+from mbo_utilities.metadata import get_voxel_size
+from mbo_utilities.util import load_npy
+
 
 def _normalize_iscell(iscell):
     """Ensure iscell is 1D boolean array."""
@@ -57,25 +60,6 @@ def _load_plane_data(plane_dir, iscell=None, stat=None, ops=None):
     return iscell, stat, ops, plane_dir
 
 
-def _get_pixel_size(ops):
-    """Extract pixel size in microns from ops dictionary."""
-    if ops is None:
-        return None
-
-    if "umPerPix" in ops:
-        return ops["umPerPix"]
-    elif "um_per_pixel" in ops:
-        return ops["um_per_pixel"]
-    elif "pixel_resolution" in ops and ops["pixel_resolution"]:
-        pr = ops["pixel_resolution"]
-        if isinstance(pr, (list, tuple)) and len(pr) >= 2:
-            return np.mean([pr[0], pr[1]])
-        return float(pr)
-    elif "umPerPixX" in ops and "umPerPixY" in ops:
-        return np.mean([ops["umPerPixX"], ops["umPerPixY"]])
-    return None
-
-
 def _save_filtered_iscell(plane_dir, iscell_filtered, iscell_original=None):
     """
     Save filtered iscell to plane_dir, preserving probability column.
@@ -269,9 +253,10 @@ def filter_by_max_diameter(
         raise ValueError("Must specify at least one of: max_diameter_um, max_diameter_px, "
                          "min_diameter_um, min_diameter_px")
 
-    # Get pixel size for unit conversion
-    if pixel_size_um is None:
-        pixel_size_um = _get_pixel_size(ops)
+    # get pixel size for unit conversion
+    if pixel_size_um is None and ops is not None:
+        voxel = get_voxel_size(ops)
+        pixel_size_um = (voxel.dx + voxel.dy) / 2 if voxel.dx != 1.0 or voxel.dy != 1.0 else None
 
     # Get radii from stat
     if "radius" not in stat[0]:
@@ -619,6 +604,7 @@ def apply_filters(
         total_removed |= removed
         filter_results.append({
             "name": name,
+            "config": config,  # original user params (without 'name' and 'save')
             "removed_mask": removed,
             "info": info,
         })
@@ -846,18 +832,17 @@ def dff_rolling_percentile(
     Notes
     -----
     Window size recommendations:
-    - Baseline window (~10 × tau × fs): Should span multiple transients so the
+
+    * Baseline window (~10 × tau × fs): Should span multiple transients so the
       percentile filter can find baseline between events.
-    - Smooth window (~0.5 × tau × fs): Should be shorter than typical transients
+    * Smooth window (~0.5 × tau × fs): Should be shorter than typical transients
       to preserve them while averaging out noise.
 
-    For GCaMP6s (tau ≈ 1.0s) at 30 Hz:
-    - window_size ≈ 300 frames (10 seconds)
-    - smooth_window ≈ 15 frames (0.5 seconds)
+    For GCaMP6s (tau ~ 1.0s) at 30 Hz: window_size ~ 300 frames (10 seconds),
+    smooth_window ~ 15 frames (0.5 seconds).
 
-    For GCaMP6f (tau ≈ 0.4s) at 30 Hz:
-    - window_size ≈ 120 frames (4 seconds)
-    - smooth_window ≈ 6 frames (0.2 seconds)
+    For GCaMP6f (tau ~ 0.4s) at 30 Hz: window_size ~ 120 frames (4 seconds),
+    smooth_window ~ 6 frames (0.2 seconds).
     """
     from scipy.ndimage import uniform_filter1d
 
@@ -1057,12 +1042,12 @@ def compute_trace_quality_score(
     baseline = np.maximum(baseline, 1e-6)
     dff = (F_corr - baseline) / baseline
 
-    # --- SNR ---
+    # SNR
     signal = np.std(dff, axis=1)
     noise = np.median(np.abs(np.diff(dff, axis=1)), axis=1) / 0.6745
     snr = signal / (noise + 1e-6)
 
-    # --- Skewness ---
+    # Skewness
     if stat is not None:
         # Use pre-computed skewness from Suite2p stat
         skewness = np.array([s.get('skew', np.nan) for s in stat])
@@ -1074,10 +1059,10 @@ def compute_trace_quality_score(
         # Compute from traces
         skewness = skew(dff, axis=1)
 
-    # --- Shot noise ---
+    # Shot noise
     shot_noise = dff_shot_noise(dff, fs)
 
-    # --- Normalize metrics to z-scores ---
+    # Normalize metrics to z-scores
     def safe_zscore(x):
         """Z-score with handling for constant arrays."""
         std = np.nanstd(x)
@@ -1090,7 +1075,7 @@ def compute_trace_quality_score(
     # Invert shot noise (lower noise = higher score)
     shot_noise_z = -safe_zscore(shot_noise)
 
-    # --- Compute weighted score ---
+    # Compute weighted score
     score = (
         weights['snr'] * snr_z +
         weights['skewness'] * skewness_z +
@@ -1161,6 +1146,132 @@ def sort_traces_by_quality(
     return F_sorted, sort_idx, quality
 
 
+def compute_roi_stats(plane_dir, fs=None):
+    """
+    Compute per-ROI statistics and save to roi_stats.npy.
+
+    Computes SNR, skewness, shot noise, mean fluorescence, and other metrics
+    for each ROI in a plane directory.
+
+    Parameters
+    ----------
+    plane_dir : str or Path
+        Path to suite2p plane directory containing F.npy, Fneu.npy, stat.npy, iscell.npy.
+    fs : float, optional
+        Frame rate in Hz. If None, reads from ops.npy.
+
+    Returns
+    -------
+    Path
+        Path to saved roi_stats.npy file.
+
+    Notes
+    -----
+    Saves a structured numpy array with fields:
+    - roi_id: ROI index
+    - accepted: whether ROI is classified as cell (from iscell)
+    - snr: signal-to-noise ratio
+    - skew: skewness of dF/F trace
+    - shot_noise: frame-to-frame noise level
+    - mean_f: mean raw fluorescence
+    - std_f: std of raw fluorescence
+    - mean_dff: mean dF/F
+    - std_dff: std of dF/F
+    - npix: number of pixels in ROI
+    - compact: compactness metric from suite2p
+    - radius: estimated radius from suite2p
+    """
+    from scipy.stats import skew
+
+    plane_dir = Path(plane_dir)
+    if plane_dir.suffix == ".npy":
+        plane_dir = plane_dir.parent
+
+    # load required files
+    F = np.load(plane_dir / "F.npy", allow_pickle=True)
+    Fneu = np.load(plane_dir / "Fneu.npy", allow_pickle=True)
+    stat = np.load(plane_dir / "stat.npy", allow_pickle=True)
+    iscell = np.load(plane_dir / "iscell.npy", allow_pickle=True)
+
+    if fs is None:
+        ops_file = plane_dir / "ops.npy"
+        if ops_file.exists():
+            ops = np.load(ops_file, allow_pickle=True).item()
+            fs = ops.get("fs", 30.0)
+        else:
+            fs = 30.0
+
+    n_rois = F.shape[0]
+
+    # neuropil correction and dF/F
+    F_corr = F - 0.7 * Fneu
+    baseline = np.percentile(F_corr, 20, axis=1, keepdims=True)
+    baseline = np.maximum(baseline, 1e-6)
+    dff = (F_corr - baseline) / baseline
+
+    # compute metrics
+    signal = np.std(dff, axis=1)
+    noise = np.median(np.abs(np.diff(dff, axis=1)), axis=1) / 0.6745
+    snr = signal / (noise + 1e-6)
+
+    skewness = np.array([s.get("skew", np.nan) for s in stat])
+    nan_mask = np.isnan(skewness)
+    if nan_mask.any():
+        skewness[nan_mask] = skew(dff[nan_mask], axis=1)
+
+    shot_noise = dff_shot_noise(dff, fs)
+
+    mean_f = np.mean(F, axis=1)
+    std_f = np.std(F, axis=1)
+    mean_dff = np.mean(dff, axis=1)
+    std_dff = np.std(dff, axis=1)
+
+    npix = np.array([s.get("npix", 0) for s in stat])
+    compact = np.array([s.get("compact", np.nan) for s in stat])
+    radius = np.array([s.get("radius", np.nan) for s in stat])
+
+    # iscell handling
+    if iscell.ndim == 2:
+        accepted = iscell[:, 0].astype(bool)
+    else:
+        accepted = iscell.astype(bool)
+
+    # build structured array
+    dtype = [
+        ("roi_id", "i4"),
+        ("accepted", "?"),
+        ("snr", "f4"),
+        ("skew", "f4"),
+        ("shot_noise", "f4"),
+        ("mean_f", "f4"),
+        ("std_f", "f4"),
+        ("mean_dff", "f4"),
+        ("std_dff", "f4"),
+        ("npix", "i4"),
+        ("compact", "f4"),
+        ("radius", "f4"),
+    ]
+
+    roi_stats = np.zeros(n_rois, dtype=dtype)
+    roi_stats["roi_id"] = np.arange(n_rois)
+    roi_stats["accepted"] = accepted
+    roi_stats["snr"] = snr.astype(np.float32)
+    roi_stats["skew"] = skewness.astype(np.float32)
+    roi_stats["shot_noise"] = shot_noise.astype(np.float32)
+    roi_stats["mean_f"] = mean_f.astype(np.float32)
+    roi_stats["std_f"] = std_f.astype(np.float32)
+    roi_stats["mean_dff"] = mean_dff.astype(np.float32)
+    roi_stats["std_dff"] = std_dff.astype(np.float32)
+    roi_stats["npix"] = npix.astype(np.int32)
+    roi_stats["compact"] = compact.astype(np.float32)
+    roi_stats["radius"] = radius.astype(np.float32)
+
+    save_path = plane_dir / "roi_stats.npy"
+    np.save(save_path, roi_stats)
+
+    return save_path
+
+
 def load_planar_results(ops: dict | str | Path, z_plane: list | int = None) -> dict:
     """
     Load stat, iscell, spks files and return as a dict. Does NOT filter by valid cells, arrays contain both
@@ -1237,9 +1348,32 @@ def load_planar_results(ops: dict | str | Path, z_plane: list | int = None) -> d
 
 
 def load_ops(ops_input: str | Path | list[str | Path]) -> dict:
-    """Simple utility load a suite2p npy file"""
+    """
+    Load a Suite2p ops.npy file.
+
+    Uses cross-platform loader to handle ops.npy files created on different
+    operating systems (Windows vs Linux/Mac Path objects in pickled data).
+
+    Parameters
+    ----------
+    ops_input : str, Path, or dict
+        Path to ops.npy file, directory containing ops.npy, or an
+        already-loaded ops dict.
+
+    Returns
+    -------
+    dict
+        Suite2p operations dictionary containing pipeline parameters
+        and results metadata.
+    """
     if isinstance(ops_input, (str, Path)):
-        return np.load(ops_input, allow_pickle=True).item()
+        ops_path = Path(ops_input)
+        if ops_path.is_dir():
+            ops_path = ops_path / "ops.npy"
+        if not ops_path.exists():
+            raise FileNotFoundError(f"ops.npy not found: {ops_path}")
+        ops = load_npy(ops_path)
+        return ops.item() if ops.ndim == 0 else ops
     elif isinstance(ops_input, dict):
         return ops_input
     print("Warning: No valid ops file provided, returning empty dict.")
